@@ -1,5 +1,5 @@
-/****************************************************************************************[Solver.C]
-ManySAT1.0 -- Copyright (c) 2008-2009, based on Minisat2.0
+/****************************************************************************************[Solver.h]
+MiniSat -- Copyright (c) 2003-2006, Niklas Een, Niklas Sorensson
 
 Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
 associated documentation files (the "Software"), to deal in the Software without restriction,
@@ -38,6 +38,12 @@ OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWA
 #include <vector>
 #include <cmath>
 
+
+struct Bclause{
+  Lit a;
+  Clause * cl;
+};
+
 //=================================================================================================
 // Solver -- the main class:
 class Cooperation;
@@ -58,9 +64,10 @@ public:
 	// Multi-threading
     int threadId; //omp rank useful for control flow
     int nbExportedClauses;
-    int nbImportedClauses;
-    int nbImportedUnits;
     int nbExportedUnits;
+    int totNbExportedUnits;
+    int totNbExportedClauses;
+
     void printVarActivity();
 
     // Solving:
@@ -135,7 +142,7 @@ public:
     double              cla_inc;          // Amount to bump next clause with.
     vec<double>         activity;         // A heuristic measurement of the activity of a variable.
     double              var_inc;          // Amount to bump next variable with.
-    vec<vec<Clause*> >  watches;          // 'watches[lit]' is a list of constraints watching 'lit' (will go there if literal becomes true).
+    vec<vec<Bclause> >  watches;          // 'watches[lit]' is a list of constraints watching 'lit' (will go there if literal becomes true).
     vec<char>           assigns;          // The current assignments (lbool:s stored as char:s).
     vec<char>           polarity;         // The preferred polarity of each variable.
     vec<char>           decision_var;     // Declares if a variable is eligible for selection in the decision heuristic.
@@ -155,15 +162,41 @@ public:
     // Temporaries (to reduce allocation overhead). Each variable is prefixed by the method in which it is
     // used, exept 'seen' wich is used in several places.
     //
-    
-    int                 tottreeWidth;
-    int                 totbackT;
-    int                 nbstFix;
-    int                 nbst;
-    double*             backT;
-    double*             treeWidth;
-    Lit                 conflict_lit;
-    vec<Lit>            GraphLit;
+    /*double  AverageSzTree;
+    double  AverageSzTreeOld;
+    double  AverageJump;
+    int     NbConfInRest;
+    int     nb_res;
+    double  AverageJumpOld;
+    int     b1, b2;
+    int     Res;
+    bool    eraze;
+    int     tt;
+    int     nb_tt;*/
+    int     prior_limit;
+    vec<Lit> prior_lits;
+    double max_act;
+    int nbConf;
+    int * receivedFrom;
+    int * keepedFrom;
+    int nbn;
+    int nb_tt;
+    double res_min;
+    double res_max;
+    int Res;
+    vec<Lit> GraphLit;
+    int tt;
+    bool eraze;
+    int b1;
+    int b2;
+    double n1;
+    double n1_old;
+    double n2;
+    int m1;
+    int nb_res;
+    double somme;
+    int nb_unit;
+    Lit    conflict_lit;
     vec<Lit>            Graph_Lit;
     vec<char>           seen2;
     vec<char>           seen3;
@@ -175,23 +208,26 @@ public:
 
     // Main internal methods:
     //
-    void     ChooseRestart(int conflictC);
-    void     init();
+    void     searchMostActivVar(Cooperation* C);
+    void     Clauses_shared_Control(Cooperation * coop);
+    void     Clauses_shared_Control_Q(Cooperation * coop);
+    void     initFirstProGroup();
     void     insertVarOrder   (Var x);                                                 // Insert a variable in the decision order priority queue.
-    Lit      pickBranchLit    (int polarity_mode, double random_var_freq);             // Return the next decision variable.
+    Lit      pickBranchLit    (int polarity_mode, double random_var_freq, Cooperation* C);             // Return the next decision variable.
     void     newDecisionLevel (); 
     void     uncheckedEnqueue (Lit p, Clause* from = NULL);                            // Enqueue a literal. Assumes value of literal is undefined.
     bool     enqueue          (Lit p, Clause* from = NULL);                            // Test if fact 'p' contradicts current state, enqueue otherwise.
     Clause*  propagate        ();                                                      // Perform unit propagation. Returns possibly conflicting clause.
     void     cancelUntil      (int level);                                             // Backtrack until a certain level.
     void     analyze          (Clause* confl, vec<Lit>& out_learnt, int& out_btlevel); // (bt = backtrack)
+    void     Ranalyze          (Clause* confl, vec<Lit>& out_learnt, int& out_btlevel); // (bt = backtrack)
     void     analyzeFinal     (Lit p, vec<Lit>& out_conflict);                         // COULD THIS BE IMPLEMENTED BY THE ORDINARIY "analyze" BY SOME REASONABLE GENERALIZATION?
     bool     litRedundant     (Lit p, uint32_t abstract_levels);                       // (helper method for 'analyze()')
     lbool    search           (int nof_conflicts, int nof_learnts, Cooperation* coop);                    // Search for a given number of conflicts.
     bool      searchReasonFromSuccess(vec<Lit>& out_learnt,int & out_btlevel, vec<Lit>& learned);
     void      NewVarBump(Lit assert_lit);
     void     New_restart(int threadId);
-    void     reduceDB         ();                                                      // Reduce the set of learnt clauses.
+    void     reduceDB         (Cooperation* C);                                                      // Reduce the set of learnt clauses.
     void     removeSatisfied  (vec<Clause*>& cs);                                      // Remove satisfied clauses from the list 'cs'.
 
     // Maintaining Variable/Clause activity:
@@ -227,7 +263,7 @@ public:
 
     // Returns a random float 0 <= x < 1. Seed must never be 0.
     static inline double drand(int threadId,double& seed) {
-      seed = seed / (threadId +1);//yh
+      seed = seed / (threadId + 1);//yh
       seed *= 1389796;
       int q = (int)(seed / 2147483647);
       seed -= (double)q * 2147483647;
@@ -248,11 +284,16 @@ inline void Solver::insertVarOrder(Var x) {
 inline void Solver::varDecayActivity() { var_inc *= var_decay; }
 
 inline void Solver::varBumpActivity(Var v) {
+
+  if(activity[v] + var_inc > max_act)
+    max_act = activity[v] + var_inc;
+   
     if ( (activity[v] += var_inc) > 1e100 ) {
-        // Rescale:
-        for (int i = 0; i < nVars(); i++)
-            activity[i] *= 1e-100;
-        var_inc *= 1e-100; } 
+      max_act *= 1e-100;
+      // Rescale:
+      for (int i = 0; i < nVars(); i++)
+	activity[i] *= 1e-100;
+      var_inc *= 1e-100; } 
 
     // Update order_heap with respect to new activity:
     if (order_heap.inHeap(v))
@@ -331,47 +372,64 @@ inline void Solver::printClause(const C& c)
     }
 }
 
-//=================================================================================================
-// Cooperation -- the main class:
+/////////////////////////////////////////////////////
+//		Multi-threading Search
+/////////////////////////////////////////////////////
 
 class Cooperation{
- public:
-  int           nbThreads;
-  Solver*       solvers;
-  bool          End;
-  bool          start;
-  int           restarts;
-  int           limitExportClauses;
-  
-  Lit***         extraClauses;
-  int**          extraClausesSizes;
-  Lit**          extraUnits;
-  int*           nbExtraUnits;
-	         
-  int*           nbExtraClauses;
-  int            nbExported;
-  	         
-  int            nbBests;
-  Lit**          bests;
-  	         
-  Cooperation(int, int);
-  ~Cooperation();
-  
-  void           exportClause(Solver*, vec<Lit>&);
-  void           importClauses(Solver*);
-  	         
-  void           exportUnit(Solver*, Lit unit);
-  void           importUnits(Solver*);
-	         
-  void           exportMostInterestingClauses(Solver*);
-  void           print();
+public:
+	int nbThreads;
+	Solver* solvers;
+	bool End;
+	bool start;
+	int restarts;
+	int     limitExportClauses;
+	
+	void addC(Solver*, Lit* l, int t);
+	void addC_Q(Solver*, Lit* l, int t);
+	Lit****  extraClauses;
+	int**    headExtraClauses;
+	int**    tailExtraClauses;
 
-  uint64_t       nbsharedUnitClauses;
-  uint64_t       nbsharedClauses;
-  	         
-  	         
+	Lit***   extraUnits;
+	int**    headExtraUnits;
+	int**    tailExtraUnits;
+	
+	int      aimd;
 
+	double ** e;  //aimd parameters                                                                                         
+        double ** quality;  //aimd parameters                                                                         
+        double a;
+        double b;
+        int    periodic_Estimate_Length; // quality check periodicity                                                            
+        double    B; // throughput                          
+        double ** average_e;
+        double * nb_compute_e;
+
+
+
+	Cooperation(int, int, int);
+	void initRand();
+	void initRestarts();
+
+	int restartIncrease();
+
+	void exportClause(Solver*, vec<Lit>&);
+	void exportMostInterestingClauses(Solver*);
+	void importClauses(Solver*);
+
+	void exportUnit(Solver*, Lit unit);
+	void importUnits(Solver*);
+
+	void print();
+
+	~Cooperation(){//TODO
+	}
 };
 
+
+
+
+
 //=================================================================================================
 #endif
